{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE NoMonomorphismRestriction #-}
--module SqrParser(pSqrProgram, showResult) where
module VarFileParser where
import Text.Parsec
import Text.Parsec.Pos
import Text.Parsec.Error

import Control.Applicative ((<*>), (*>), (<*), (<$>), liftA2)
import Control.Monad
import Control.Monad.Trans
import Data.Char
import qualified Data.Map as M
import Data.List
import Data.Maybe
import SqrppParser

run s = runParserT pVarFile () "" s >>= either (print) (print)

lexeme p = do r <- p
              skipMany nbsp
              return r

pVarFile = spaces >> many pVarDefLine

pVarDefLine = (pGlobalVarDef <|> pLocalVarDef) <* skipMany (noneOf "\n\r") <* spaces

pGlobalVarDef = (,) <$> (return Global) <*> pVarName

pLocalVarDef = (,) <$> pProcName <*> pVarName

pVarName :: ParsecT String () IO VarIdentifier
pVarName = lexeme $ try $ do
    var_sym  <- pAnyVarSymbol
    var_name <- many1 (noneOf " \t\n\r")
    return $ VarIdentifier { varName=(fmap toUpper var_name), varSymbol=var_sym }

pAnyVarSymbol = pNumVarSymbol <|> pTextVarSymbol
pNumVarSymbol  = char '#' >> return Sharp
pTextVarSymbol = char '$' >> return Dollar

pProcName :: ParsecT String () IO Scope
pProcName = lexeme $ try $ do
    name <- many1 (noneOf " \t\n\r")
    skipMany nbsp
    return $ Local (fmap toUpper name)
